@@ -998,59 +998,59 @@ gdk_pixbuf_from_xcursor_image (XcursorImage *cursor) {
   return pixbuf;
 }
 
-
 static void
-look_for_cursor_theme (const gchar *theme_dir)
+read_cursor_theme (const gchar *theme_dir)
 {
-  gchar *cursors_dir;
-  gchar *cursor_theme_file;
-  GnomeDesktopItem *cursor_theme_ditem;
+  gchar *name;
+  gint i;
+  GArray *available_sizes;
+  XcursorImage *cursor;
+  GdkPixbuf *thumbnail = NULL;
+  GnomeThemeCursorInfo *cursor_theme_info;
+  const gint sizes[] = { 12, 16, 24, 32, 36, 40, 48, 64 };
+  const gint num_sizes = G_N_ELEMENTS (sizes);
 
-  cursors_dir = g_build_filename (theme_dir, "cursors", NULL);
+  name = g_path_get_basename (theme_dir);
 
-  if (g_file_test (cursors_dir, G_FILE_TEST_IS_DIR)) {
-    gchar *name;
-    gint i;
-    GArray *available_sizes;
-    XcursorImage *cursor;
-    GdkPixbuf *thumbnail = NULL;
-    GnomeThemeCursorInfo *cursor_theme_info;
-    const gint sizes[] = { 12, 16, 24, 32, 36, 40, 48, 64, 0 };
+  available_sizes = g_array_sized_new (FALSE, FALSE, sizeof (gint),
+				       num_sizes);
 
-    name = g_path_get_basename (theme_dir);
+  for (i = 0; i < num_sizes; i++) {
+    cursor = XcursorLibraryLoadImage ("left_ptr", name, sizes[i]);
 
-    available_sizes = g_array_sized_new (FALSE, FALSE, sizeof (gint),
-					 G_N_ELEMENTS (sizes) - 1);
+    if (cursor) {
+      if (cursor->size == sizes[i]) {
+        g_array_append_val (available_sizes, sizes[i]);
 
-    for (i = 0; sizes[i] != 0; i++) {
-      cursor = XcursorLibraryLoadImage ("left_ptr", name, sizes[i]);
-
-      if (cursor) {
-        if (cursor->size == sizes[i]) {
-          g_array_append_val (available_sizes, sizes[i]);
-
-          if (thumbnail == NULL && i >= 1)
-            thumbnail = gdk_pixbuf_from_xcursor_image (cursor);
-        }
-
-        XcursorImageDestroy (cursor);
+        if (thumbnail == NULL && i >= 1)
+          thumbnail = gdk_pixbuf_from_xcursor_image (cursor);
       }
-    }
 
-    if (!thumbnail) {
-      cursor = XcursorLibraryLoadImage ("left_ptr", name, g_array_index (available_sizes, gint, 0));
-      if (cursor) {
-        thumbnail = gdk_pixbuf_from_xcursor_image (cursor);
-        XcursorImageDestroy (cursor);
-      }
+      XcursorImageDestroy (cursor);
     }
+  }
 
-    cursor_theme_info = gnome_theme_cursor_info_new ();
-    cursor_theme_info->path = g_strdup (theme_dir);
-    cursor_theme_info->name = name;
-    cursor_theme_info->sizes = available_sizes;
-    cursor_theme_info->thumbnail = thumbnail;
-    cursor_theme_info->priority = 0;
+  if (!thumbnail && available_sizes->len != 0) {
+    cursor = XcursorLibraryLoadImage ("left_ptr", name,
+				      g_array_index (available_sizes, gint, 0));
+    if (cursor) {
+      thumbnail = gdk_pixbuf_from_xcursor_image (cursor);
+      XcursorImageDestroy (cursor);
+    }
+  }
+
+  cursor_theme_info = gnome_theme_cursor_info_new ();
+  cursor_theme_info->path = g_strdup (theme_dir);
+  cursor_theme_info->name = name;
+  cursor_theme_info->sizes = available_sizes;
+  cursor_theme_info->thumbnail = thumbnail;
+  cursor_theme_info->priority = 0;
+
+  if (!strcmp (name, "default")) {
+    cursor_theme_info->readable_name = g_strdup (_("Default Pointer"));
+  } else {
+    gchar *cursor_theme_file;
+    GnomeDesktopItem *cursor_theme_ditem;
 
     cursor_theme_file = g_build_filename (theme_dir, "index.theme", NULL);
     cursor_theme_ditem = gnome_desktop_item_new_from_file (cursor_theme_file, 0, NULL);
@@ -1059,7 +1059,7 @@ look_for_cursor_theme (const gchar *theme_dir)
     if (cursor_theme_ditem) {
       const gchar *readable_name;
 
-      readable_name = (gchar *) gnome_desktop_item_get_string (cursor_theme_ditem, "Icon Theme/Name");
+      readable_name = gnome_desktop_item_get_string (cursor_theme_ditem, "Icon Theme/Name");
 
       if (readable_name)
         cursor_theme_info->readable_name = g_strdup (readable_name);
@@ -1070,10 +1070,18 @@ look_for_cursor_theme (const gchar *theme_dir)
     } else {
       cursor_theme_info->readable_name = g_strdup (name);
     }
-
-    g_hash_table_insert (cursor_theme_hash_by_uri, cursor_theme_info->path, cursor_theme_info);
-    add_data_to_hash_by_name (cursor_theme_hash_by_name, name, cursor_theme_info);
   }
+  g_hash_table_insert (cursor_theme_hash_by_uri, cursor_theme_info->path, cursor_theme_info);
+  add_data_to_hash_by_name (cursor_theme_hash_by_name, name, cursor_theme_info);
+}
+
+static void
+look_for_cursor_theme (const gchar *theme_dir)
+{
+  gchar *cursors_dir = g_build_filename (theme_dir, "cursors", NULL);
+
+  if (g_file_test (cursors_dir, G_FILE_TEST_IS_DIR) || g_str_has_suffix (theme_dir, "default"))
+    read_cursor_theme (theme_dir);
 
   g_free (cursors_dir);
 }
@@ -1502,7 +1510,8 @@ gnome_theme_cursor_info_free (GnomeThemeCursorInfo *cursor_theme_info)
   g_free (cursor_theme_info->readable_name);
   g_free (cursor_theme_info->path);
   g_array_free (cursor_theme_info->sizes, TRUE);
-  g_object_unref (cursor_theme_info->thumbnail);
+  if (cursor_theme_info->thumbnail != NULL)
+    g_object_unref (cursor_theme_info->thumbnail);
   g_free (cursor_theme_info);
 }
 
@@ -1854,7 +1863,7 @@ gnome_theme_color_scheme_parse (const gchar *scheme, GdkColor *colors)
 
   /* initialise the array */
   for (i = 0; i < NUM_SYMBOLIC_COLORS; i++)
-    gdk_color_parse ("#000", &colors[i]);
+    colors[i].red = colors[i].green = colors[i].blue = 0;
 
   /* The color scheme string consists of name:color pairs, seperated by
    * newlines, so first we split the string up by new line */
